@@ -4,7 +4,7 @@
  *              to build a moderately scalable and highly available server
  *              based on a cluster of servers, with more flexibility.
  *
- * Version:     $Id: tcp_vs.c,v 1.11 2003/02/14 09:21:32 wensong Exp $
+ * Version:     $Id: tcp_vs.c,v 1.12 2003/02/22 04:51:57 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *
@@ -49,7 +49,6 @@ EXPORT_SYMBOL(tcp_vs_connect2dest);
 EXPORT_SYMBOL(tcp_vs_sendbuffer);
 EXPORT_SYMBOL(tcp_vs_xmit);
 EXPORT_SYMBOL(tcp_vs_recvbuffer);
-/*  EXPORT_SYMBOL(tcp_vs_lookup_dest); */
 EXPORT_SYMBOL(tcp_vs_getword);
 EXPORT_SYMBOL(tcp_vs_getline);
 EXPORT_SYMBOL(tcp_vs_exp_exec);
@@ -57,6 +56,11 @@ EXPORT_SYMBOL(tcp_vs_exp_exec);
 EXPORT_SYMBOL(tcp_vs_get_debug_level);
 #endif
 EXPORT_SYMBOL(sysctl_ktcpvs_unload);
+EXPORT_SYMBOL(sysctl_ktcpvs_keepalive_timeout);
+EXPORT_SYMBOL(tcp_vs_srvconn_get);
+EXPORT_SYMBOL(tcp_vs_srvconn_put);
+EXPORT_SYMBOL(tcp_vs_srvconn_new);
+EXPORT_SYMBOL(tcp_vs_srvconn_free);
 
 
 struct tcp_vs_conn *
@@ -301,7 +305,7 @@ int tcp_vs_conn_handle(struct tcp_vs_conn *conn, struct tcp_vs_service *svc)
 		TCP_VS_ERR("no destination available\n");
 		return -1;
 	default:
-		return -1;
+		return 0;
 	}
 
 	dsock = conn->dsock;
@@ -501,8 +505,8 @@ static int tcp_vs_child(void *__child)
 	struct tcp_vs_child *chd = (struct tcp_vs_child *)__child;
 	struct tcp_vs_service *svc = chd->svc;
 
-	DECLARE_WAIT_QUEUE_HEAD(queue);
-	/* DECLARE_WAITQUEUE(wait, current); */
+	/* DECLARE_WAIT_QUEUE_HEAD(queue); */
+	DECLARE_WAITQUEUE(wait, current);
 
 	EnterFunction(3);
 
@@ -544,14 +548,12 @@ static int tcp_vs_child(void *__child)
 
 		update_child_status(chd, SERVER_READY);
 		if (sock->sk->tp_pinfo.af_tcp.accept_queue == NULL) {
-#if 0000
+			/* interruptible_sleep_on_timeout(&queue, HZ); */
+			add_wait_queue(sock->sk->sleep, &wait);
 			__set_current_state(TASK_INTERRUPTIBLE);
-			schedule();
+			schedule_timeout(HZ);
 			__set_current_state(TASK_RUNNING);
-#endif
-			/* add_wait_queue(sock->sk->sleep, &wait); */
-			interruptible_sleep_on_timeout(&queue, HZ);
-			/* remove_wait_queue(sock->sk->sleep, &wait); */
+			remove_wait_queue(sock->sk->sleep, &wait);
 			continue;
 		}
 
@@ -646,7 +648,7 @@ static int tcp_vs_daemon(void *__svc)
 		/* dynamically keep enough thread to handle load */
 		child_pool_maintenance(child_table, svc);
 
-		/* reap the daemons */
+		/* reap the zombie daemons */
 		waitpid_result = waitpid(-1, NULL, __WCLONE|WNOHANG);
 	}
 
@@ -654,7 +656,7 @@ static int tcp_vs_daemon(void *__svc)
 	while (atomic_read(&svc->childcount) > 0)
 		interruptible_sleep_on_timeout(&WQ, HZ);
 
-	/* reap the zombie-daemons */
+	/* reap the zombie daemons */
 	waitpid_result = 1;
 	while (waitpid_result > 0)
 		waitpid_result = waitpid(-1, NULL, __WCLONE|WNOHANG);
@@ -704,6 +706,9 @@ static int master_daemon(void *unused)
 		}
 		read_unlock(&__tcp_vs_svc_lock);
 
+		/* run the slowtimer collection */
+		tcp_vs_slowtimer_collect();
+
 		if (signal_pending(current))
 			break;
 
@@ -718,7 +723,7 @@ static int master_daemon(void *unused)
 	while (atomic_read(&tcp_vs_daemon_count) > 0)
 		interruptible_sleep_on_timeout(&WQ, HZ);
 
-	/* reap the zombie-daemons */
+	/* reap the zombie daemons */
 	waitpid_result = 1;
 	while (waitpid_result > 0)
 		waitpid_result = waitpid(-1, NULL, __WCLONE|WNOHANG);
@@ -739,6 +744,10 @@ static int __init ktcpvs_init(void)
 {
 	tcp_vs_control_start();
 
+	tcp_vs_slowtimer_init();
+
+	tcp_vs_srvconn_init();
+
 	(void)kernel_thread(master_daemon, NULL, 0);
 
 	TCP_VS_INFO("ktcpvs loaded.\n");
@@ -746,12 +755,19 @@ static int __init ktcpvs_init(void)
 	return 0;
 }
 
+
 static void __exit ktcpvs_cleanup(void)
 {
+	tcp_vs_srvconn_cleanup();
+
+	tcp_vs_slowtimer_cleanup();
+
 	tcp_vs_control_stop();
+
 	TCP_VS_INFO("ktcpvs unloaded.\n");
 }
 
+
 module_init(ktcpvs_init);
 module_exit(ktcpvs_cleanup);
 MODULE_LICENSE("GPL");
