@@ -4,7 +4,7 @@
  *              to build a moderately scalable and highly available server
  *              based on a cluster of servers, with more flexibility.
  *
- * Version:     $Id: tcp_vs_ctl.c,v 1.9 2002/05/11 08:23:38 wensong Exp $
+ * Version:     $Id: tcp_vs_ctl.c,v 1.10 2003/02/22 04:51:57 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *
@@ -62,6 +62,7 @@ rwlock_t __tcp_vs_svc_lock = RW_LOCK_UNLOCKED;
 int sysctl_ktcpvs_unload = 0;
 int sysctl_ktcpvs_max_backlog = 2048;
 int sysctl_ktcpvs_zerocopy_send = 0;
+int sysctl_ktcpvs_keepalive_timeout = 30;
 
 #ifdef CONFIG_TCP_VS_DEBUG
 static int sysctl_ktcpvs_debug_level = 0;
@@ -1029,6 +1030,9 @@ static ctl_table ktcpvs_table[] = {
 	{NET_KTCPVS_ZEROCOPY_SEND, "zerocopy_send",
 	 &sysctl_ktcpvs_zerocopy_send,
 	 sizeof(int), 0644, NULL, &proc_dointvec},
+	{NET_KTCPVS_KEEPALIVE_TIMEOUT, "keepalive_timeout",
+	 &sysctl_ktcpvs_keepalive_timeout,
+	 sizeof(int), 0644, NULL, &proc_dointvec},
 	{0}
 };
 
