@@ -7,7 +7,7 @@
  * tcp_vs_phttp.c: KTCPVS content-based scheduling module for HTTP service
  *		   with persistent connection support
  *
- * Version:     $Id: tcp_vs_phttp.c,v 1.1 2003/02/14 09:21:32 wensong Exp $
+ * Version:     $Id: tcp_vs_phttp.c,v 1.2 2003/02/22 04:51:57 wensong Exp $
  *
  * Authors:     Wensong Zhang, <wensong@linuxvirtualserver.org>
  *              Hai Long, <david_lung@yahoo.com>
@@ -39,8 +39,6 @@
 #define LF 10
 #define SP ' '
 
-#define KEEP_ALIVE_TIMES	60
-
 #define PARSE_OK		0
 #define PARSE_ERROR		1
 
@@ -152,12 +150,6 @@ typedef struct http_read_ctl_block_s
 	int		flag;		/* read flag */
 } http_read_ctl_block_t;
 
-typedef struct http_dest_server_s {
-	struct list_head	d_list;	/* server list for a client connection */
-	struct tcp_vs_dest	*dest;	/* dest server entry */
-	struct socket		*sock;	/* destination socket */
-} http_dest_server_t;
-
 
 static int tcp_vs_phttp_init_svc(struct tcp_vs_service *svc)
 {
@@ -1191,27 +1183,33 @@ http_get_response (struct socket *csock, struct socket* dsock,
 */
 static inline int data_available (http_read_ctl_block_t *ctl_blk)
 {
-	char buf[10];
-	int ret = 0;
+	int ret;
 
-	EnterFunction(6);
+	EnterFunction(12);
 
 	if (ctl_blk->remaining == 0) {
+		/* check if the connection is closed */
+		if (ctl_blk->sock->sk->state != TCP_ESTABLISHED
+		    && ctl_blk->sock->sk->state != TCP_CLOSE_WAIT) {
+			ret = -1;
+			goto out;
+		}
+
 		/* Do we have data ? */
 		if (skb_queue_empty(&(ctl_blk->sock->sk->receive_queue)))
-			goto out;
-
-		ret = tcp_vs_recvbuffer(ctl_blk->sock, buf, 10, MSG_PEEK);
-		if (ret > 0)
+			ret = 0;
+		else
 			ret = 1;
-	}
+		goto out;
+	} else
+		ret = 1;
 
   out:
-	LeaveFunction(6);
+	LeaveFunction(12);
 	return ret;
-
 }
 
+
 /****************************************************************************
 *	HTTP content-based scheduling:
 *	1, For http 1.0 request, parse the http request, select a server
@@ -1235,13 +1233,12 @@ static int tcp_vs_phttp_schedule(struct tcp_vs_conn *conn, struct tcp_vs_service
 	int			len;
 	unsigned long		last_read;
 	int			close_server = 0;
-	http_dest_server_t	*server = NULL;
-	struct list_head	dest_list;	/* destination server list */
-	struct list_head	*l;
 	struct tcp_vs_dest	*dest;
 	struct socket		*dsock;
+	server_conn_t		*sc;
 
-	DECLARE_WAIT_QUEUE_HEAD(WQ);
+	/* DECLARE_WAIT_QUEUE_HEAD(WQ); */
+	DECLARE_WAITQUEUE(wait, current);
 
 	EnterFunction(5);
 
@@ -1254,11 +1251,9 @@ static int tcp_vs_phttp_schedule(struct tcp_vs_conn *conn, struct tcp_vs_service
 	conn->dest	= NULL;
 	conn->dsock	= NULL;
 
-	INIT_LIST_HEAD(&dest_list);
-
 	if (read_ctl_blk.len < HTTP_MAX_REQLINE_SIZE) {
 		TCP_VS_ERR("Error: buffer is too small!\n");
-		goto exit;
+		goto out;
 	}
 
 	/* Do we have data ? */
@@ -1269,7 +1264,7 @@ static int tcp_vs_phttp_schedule(struct tcp_vs_conn *conn, struct tcp_vs_service
 	/* allocate buffer to store data that get from servers */
 	buffer = (char*) get_free_page(GFP_KERNEL);
 	if (buffer == NULL) {
-		goto exit;
+		goto out;
 	}
 
 	last_read = jiffies;
@@ -1278,7 +1273,7 @@ static int tcp_vs_phttp_schedule(struct tcp_vs_conn *conn, struct tcp_vs_service
 		case -1:
 			TCP_VS_DBG(5, "Socket error before reading request line.\n");
 			ret = -2;
-			goto done;
+			goto out;
 
 		case 0:
 			/* check if the service is stopped or system is
@@ -1286,14 +1281,21 @@ static int tcp_vs_phttp_schedule(struct tcp_vs_conn *conn, struct tcp_vs_service
 			if (svc->stop != 0 || sysctl_ktcpvs_unload != 0) {
 				TCP_VS_DBG(5, "phttp scheduling exit (pid=%d)\n",
 					   current->pid);
-				goto done;
+				goto out;
 			}
 
-			interruptible_sleep_on_timeout(&WQ, HZ);
-			if ((jiffies - last_read) >  KEEP_ALIVE_TIMES*HZ) {
+			if ((jiffies - last_read) >
+			    sysctl_ktcpvs_keepalive_timeout * HZ) {
 				TCP_VS_DBG(5, "Timeout, disconnect.\n");
-				goto done;
+				goto out;
 			}
+
+			/* interruptible_sleep_on_timeout(&WQ, HZ); */
+			add_wait_queue(read_ctl_blk.sock->sk->sleep, &wait);
+			__set_current_state(TASK_INTERRUPTIBLE);
+			schedule_timeout(5*HZ);
+			__set_current_state(TASK_RUNNING);
+			remove_wait_queue(read_ctl_blk.sock->sk->sleep, &wait);
 			continue;
 
 		case 1:
@@ -1306,7 +1308,7 @@ static int tcp_vs_phttp_schedule(struct tcp_vs_conn *conn, struct tcp_vs_service
 		if (len < 0) {
 			TCP_VS_ERR("Error reading request line from client\n");
 			ret = -2;
-			goto done;
+			goto out;
 		}
 
 		/* parse the http request line */
@@ -1315,7 +1317,7 @@ static int tcp_vs_phttp_schedule(struct tcp_vs_conn *conn, struct tcp_vs_service
 		    != PARSE_OK) {
 			TCP_VS_ERR("Cannot parse http request\n");
 			ret = -2;
-			goto done;
+			goto out;
 		}
 
 		/* select a server */
@@ -1323,12 +1325,12 @@ static int tcp_vs_phttp_schedule(struct tcp_vs_conn *conn, struct tcp_vs_service
 		if (!dest) {
 			TCP_VS_DBG(5, "Can't find a right server\n");
 			if (read_ctl_blk.flag == MSG_PEEK) {
+				/* redirect to a local port */
 				ret = -1;
-				goto exit;	/* redirect to a local port */
 			} else {
 				ret = -2;
-				goto done;
 			}
+			goto out;
 		}
 
 		if (req.version <= HTTP_V_1_0) {
@@ -1341,94 +1343,75 @@ static int tcp_vs_phttp_schedule(struct tcp_vs_conn *conn, struct tcp_vs_service
 			conn->dsock = tcp_vs_connect2dest(dest);
 			if (conn->dsock == NULL) {
 				TCP_VS_ERR("The destination is not available!\n");
-				goto exit;
+				goto out;
 			}
 
 			atomic_inc(&dest->conns);
 			conn->dest = dest;
 			ret = 0;
-			goto exit;
+			goto out;
 		}
 
+
 		/*
 		 * For http 1.1 client, continue processing for
 		 * persistent connections
 		 */
 
-		/* find the dest server from server list, delete it if it's a dead one */
-		dsock = NULL;
-		list_for_each (l, &dest_list) {
-			server = list_entry (l, http_dest_server_t, d_list);
-			if (server->dest == dest) {
-				assert(server->sock != NULL);
-				if (server->sock->sk->state != TCP_ESTABLISHED
-				    && server->sock->sk->state != TCP_CLOSE_WAIT) {
-					sock_release(server->sock);
-					list_del(&server->d_list);
-					atomic_dec(&server->dest->conns);
-					kfree(server);
-				} else {
-					dsock = server->sock;
-				}
-				break;
-			}
-		}
-
-		if (dsock == NULL) {
-			/* open socket for a new server */
-			dsock = tcp_vs_connect2dest(dest);
-			if (dsock == NULL) {
-				TCP_VS_ERR("The destination is not available\n");
+		/* lookup a server connection in the connection pool */
+	  lookup_again:
+		sc = tcp_vs_srvconn_get(dest->addr, dest->port);
+		if (sc == NULL) {
+			sc = tcp_vs_srvconn_new(dest);
+			if (sc == NULL) {
 				if (read_ctl_blk.flag == MSG_PEEK) {
+					/* redirect to a local port */
 					ret = -1;
-					goto exit;	/* redirect to a local port */
 				} else {
 					ret = -2;
-					goto done;
 				}
+				goto out;
 			}
-
-			if ((server = kmalloc(sizeof(http_dest_server_t),
-					      GFP_ATOMIC)) == NULL) {
-				ret = -2;
-				goto done;
-			}
-			server->dest = dest;
-			server->sock = dsock;
-			atomic_inc(&dest->conns);
-			list_add_tail(&server->d_list, &dest_list);
+		}
+		dsock = sc->sock;
+		if (dsock->sk->state != TCP_ESTABLISHED) {
+			tcp_vs_srvconn_free(sc);
+			goto lookup_again;
 		}
 
-		/* re-read the peeked data for the first http request of a connection */
+		/* re-read the peeked data for the first http request of
+		   a connection */
 		if (read_ctl_blk.flag == MSG_PEEK) {
-			read_ctl_blk.flag		= 0;
-			read_ctl_blk.offset		= 0;
+			read_ctl_blk.flag	= 0;
+			read_ctl_blk.offset	= 0;
 			read_ctl_blk.remaining	= 0;
 			if (tcp_vs_recvbuffer(conn->csock,
 					      read_ctl_blk.buffer,
 					      len + 2, 0) != (len + 2)) {
 				TCP_VS_ERR("Error in re-reading http request line\n");
-				goto exit;
+				tcp_vs_srvconn_put(sc);
+				goto out;
 			}
 		}
 
 		/* xmit request line (2 more bytes for CRLF) */
 		if (tcp_vs_xmit(dsock, read_ctl_blk.info, len + 2, MSG_MORE) < 0) {
 			TCP_VS_ERR("Error in sending request line\n");
-			goto done;
+			goto out_free;
 		}
 
 		/* Process MIME header */
 		do {
 			len = http_read_line (&read_ctl_blk, HTTP_MAX_MIMLINE_SIZE);
 			if (len < 0) {
-				goto done;
+				tcp_vs_srvconn_put(sc);
+				goto out;
 			}
 
 			/* xmit MIME header (2 more bytes for CRLF) */
 			if (tcp_vs_xmit (dsock, read_ctl_blk.info, len + 2, MSG_MORE) < 0) {
 				TCP_VS_ERR("Error in sending MIME header\n");
-				goto done;
+				goto out_free;
 			}
 
 			http_line_unescape (read_ctl_blk.info, len);
@@ -1437,42 +1420,34 @@ static int tcp_vs_phttp_schedule(struct tcp_vs_conn *conn, struct tcp_vs_service
 
 		if (relay_http_message_body(dsock, &read_ctl_blk, &req.mime) != 0) {
 			TCP_VS_ERR("Error in sending http message body\n");
-			goto done;
+			goto out_free;
 		}
 
 		if (http_get_response(conn->csock, dsock, &req, buffer,
 				      PAGE_SIZE, &close_server) < 0) {
-			goto done;
+			goto out_free;
 		}
 
 		if (close_server) {
-			sock_release(dsock);
-			list_del (&server->d_list);
-			atomic_dec (&server->dest->conns);
-			kfree(server);
 			TCP_VS_DBG(5, "Close server connection.\n");
-			break;	/* close the connection? tbd */
+			goto out_free;	/* close the connection? tbd */
 		}
+
+		/* have to put back the server connection when
+		   it is not used */
+		tcp_vs_srvconn_put(sc);
 	} while (req.mime.connection_close != 1);
 
-  done:
-	for (l = &dest_list; l->next != l; ) {
-		server = list_entry(l->next, http_dest_server_t, d_list);
-		if (server->sock) {
-			sock_release(server->sock);
-		}
-		list_del(&server->d_list);
-		if (server->dest)
-			atomic_dec(&server->dest->conns);
-		kfree(server);
-	}
-
-  exit:
+  out:
 	if (buffer) {
 		free_page ((unsigned long)buffer);
 	}
 	LeaveFunction(5);
 	return ret;
+
+  out_free:
+	tcp_vs_srvconn_free(sc);
+	goto out;
 }
 
 
