@@ -4,7 +4,7 @@
  *              to build a moderately scalable and highly available server
  *              based on a cluster of servers, with more flexibility.
  *
- * Version:     $Id: tcp_vs_sched.c,v 1.4 2002/05/05 12:12:09 wensong Exp $
+ * Version:     $Id: tcp_vs_sched.c,v 1.5 2003/02/22 04:51:57 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *
@@ -233,7 +233,7 @@ struct tcp_vs_scheduler * tcp_vs_scheduler_get(const char *sched_name)
 	 */
 	if (sched == NULL) {
 		char module_name[KTCPVS_SCHEDNAME_MAXLEN];
-		sprintf(module_name, "tcp_vs_%s", sched_name);
+		sprintf(module_name, "ktcpvs_%s", sched_name);
 		request_module(module_name);
 		sched = tcp_vs_sched_getbyname(sched_name);
 	}
