@@ -4,7 +4,7 @@
  *              to build a moderately scalable and highly available server
  *              based on a cluster of servers, with more flexibility.
  *
- * Version:     $Id: tcp_vs_wlc.c,v 1.5 2002/05/05 12:12:09 wensong Exp $
+ * Version:     $Id: tcp_vs_wlc.c,v 1.6 2002/05/26 14:05:16 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *
@@ -47,8 +47,8 @@ static int tcp_vs_wlc_update_svc(struct tcp_vs_service *svc)
 /*
  *    Weighted Least Connection scheduling
  */
-static struct socket *tcp_vs_wlc_schedule(struct tcp_vs_conn *conn,
-					  struct tcp_vs_service *svc)
+static int
+tcp_vs_wlc_schedule(struct tcp_vs_conn *conn, struct tcp_vs_service *svc)
 {
 	register struct list_head *l, *e;
 	struct tcp_vs_dest *dest, *least;
@@ -77,7 +77,7 @@ static struct socket *tcp_vs_wlc_schedule(struct tcp_vs_conn *conn,
 		}
 	}
 	read_unlock(&svc->lock);
-	return NULL;
+	return -1;
 
 	/*
 	 *    Find the destination with the least load.
@@ -98,11 +98,15 @@ static struct socket *tcp_vs_wlc_schedule(struct tcp_vs_conn *conn,
 		   atomic_read(&least->conns),
 		   atomic_read(&least->refcnt), least->weight);
 
+	conn->dsock = tcp_vs_connect2dest(least);
+	if (!conn->dsock) {
+		TCP_VS_ERR_RL("The destination is not available\n");
+		return -1;
+	}
 	atomic_inc(&least->conns);
 	conn->dest = least;
-	conn->dsock = tcp_vs_connect2dest(least);
 
-	return conn->dsock;
+	return 0;
 }
 
 
