@@ -1,7 +1,7 @@
 /*
  *      tcpvsadm - TCP Virtual Server ADMinistration program
  *
- *      Version: $Id: tcpvsadm.c,v 1.9 2003/02/14 09:22:39 wensong Exp $
+ *      Version: $Id: tcpvsadm.c,v 1.10 2003/02/22 04:51:59 wensong Exp $
  *
  *      Authors: Wensong Zhang <wensong@linuxvirtualserver.org>
  *
@@ -50,11 +50,11 @@
 #include "libtcpvs/libtcpvs.h"
 
 static const char *program;
-#define TCPVSADM_VERSION_DATE	"19-Dec-2001"
+#define TCPVSADM_VERSION_DATE	"22-Feb-2003"
 static const char *program_version = VERSION ", " TCPVSADM_VERSION_DATE;
 
 /* default scheduler */
-#define DEF_SCHED	"wlc"
+#define DEFAULT_SCHEDULER	"http"
 
 /* check the options based on the commands_v_options table */
 static void generic_opt_check(int command, int options);
@@ -214,7 +214,7 @@ int main(int argc, char **argv)
 	conf.startservers	= 8;
 	conf.maxClients		= 256;
 	dest.weight		= 1;
-	strcpy(conf.sched_name, DEF_SCHED);
+	strcpy(conf.sched_name, DEFAULT_SCHEDULER);
 
 	/*
 	 *	Parse options
@@ -650,7 +650,7 @@ static void usage_exit(const int status)
 		"  --real-server  -r server-address    server-address is host (and port)\n"
 		"  --weight       -w weight            capacity of real server\n"
 		"  --numeric      -n                   numeric output of addresses and ports\n",
-		DEF_SCHED);
+		DEFAULT_SCHEDULER);
 
 	exit(status);
 }
